@@ -15,26 +15,29 @@
 #include "sysconfig.h"
 #endif
 
-#include "wx/wxprec.h"
+#include <wx/wxprec.h>
 
 #ifdef __BORLANDC__
 	#pragma hdrstop
 #endif
 
 #ifndef WX_PRECOMP
-	#include "wx/wx.h"
+	#include <wx/wx.h>
 #endif
 
-#include "tqsllib.h"
 
-#include "wx/treectrl.h"
+#include <wx/treectrl.h>
+
+#include "tqsllib.h"
+#include "wxutil.h"
 
 class CertTreeItemData : public wxTreeItemData {
  public:
-	explicit CertTreeItemData(tQSL_Cert cert) : _cert(cert) {}
+	explicit CertTreeItemData(tQSL_Cert cert) : _cert(cert) {path = wxEmptyString; basename = wxEmptyString;}
 	~CertTreeItemData();
 	tQSL_Cert getCert() { return _cert; }
-
+	wxString path;
+	wxString basename;
  private:
 	tQSL_Cert _cert;
 };
@@ -47,6 +50,8 @@ class CertTree : public wxTreeCtrl {
 	int Build(int flags = TQSL_SELECT_CERT_WITHKEYS, const TQSL_PROVIDER *provider = 0);
 	void OnItemActivated(wxTreeEvent& event);
 	void OnRightDown(wxMouseEvent& event);
+	void OnKeyDown(wxTreeEvent& event);
+	void SetTabTo(wxWindow*);
 	bool useContextMenu;
 #ifdef __clang__
 #pragma clang diagnostic push
@@ -64,6 +69,7 @@ class CertTree : public wxTreeCtrl {
         tQSL_Cert *_certs;
 	int _ncerts;
 	int _nissuers;
+	wxWindow* tabTo;
 	DECLARE_EVENT_TABLE()
 };
 
