@@ -19,7 +19,7 @@
 #endif
 
 // For compilers that support precompilation, includes "wx/wx.h".
-#include "wx/wxprec.h"
+#include <wx/wxprec.h>
 
 #ifdef __BORLANDC__
     #pragma hdrstop
@@ -33,6 +33,7 @@
     #include <wx/arrstr.h>
 #endif
 
+#include <string>
 
 #include "json_defs.h"
 #include "jsonval.h"
