@@ -43,12 +43,14 @@ enum {
 BEGIN_EVENT_TABLE(LocTree, wxTreeCtrl)
 	EVT_TREE_ITEM_ACTIVATED(-1, LocTree::OnItemActivated)
 	EVT_RIGHT_DOWN(LocTree::OnRightDown)
+	EVT_TREE_KEY_DOWN(wxID_ANY, LocTree::OnKeyDown)
 END_EVENT_TABLE()
 
 LocTree::LocTree(wxWindow *parent, const wxWindowID id, const wxPoint& pos,
 		const wxSize& size, long style)
 		: wxTreeCtrl(parent, id, pos, size, style), _nloc(0) {
 	tqslTrace("LocTree::LocTree", "parent=0x%lx, id=0x%lx, style=%d", reinterpret_cast<void *>(parent), reinterpret_cast<void *>(id), style);
+	tabTo = NULL;
 	useContextMenu = true;
 	wxBitmap homebm(home_xpm);
 	wxBitmap folderbm(folder_xpm);
@@ -62,6 +64,11 @@ LocTree::LocTree(wxWindow *parent, const wxWindowID id, const wxPoint& pos,
 LocTree::~LocTree() {
 }
 
+void
+LocTree::SetTabTo(wxWindow* w) {
+	tabTo = w;
+}
+
 typedef pair<wxString, int> locitem;
 typedef vector<locitem> loclist;
 
@@ -90,6 +97,8 @@ LocTree::Build(int flags, const TQSL_PROVIDER *provider) {
 	for (int i = 0; i < _nloc && i < 2000; i++) {
 		char locname[256];
 		check_tqsl_error(tqsl_getStationLocationName(loc, i, locname, sizeof locname));
+		if (!strcmp(locname, ".empty"))				// Dummy station location
+			continue;
 		char callsign[256];
 		check_tqsl_error(tqsl_getStationLocationCallSign(loc, i, callsign, sizeof callsign));
 		wxString locutf8 = wxString::FromUTF8(locname);
@@ -133,4 +142,15 @@ LocTree::OnRightDown(wxMouseEvent& event) {
 		delete cm;
 	}
 }
+void
+LocTree::OnKeyDown(wxTreeEvent& event) {
+	const wxKeyEvent &keyev = event.GetKeyEvent();
+	long keycode = keyev.GetKeyCode();
+	if (keycode == WXK_TAB) {
+		if (tabTo) {
+			tabTo->SetFocus();
+			event.Skip();
+		}
+	}
+}
 
