@@ -11,37 +11,38 @@
 #ifndef __TQSL_PREFS_H
 #define __TQSL_PREFS_H
 
+#include <map>
+#include <vector>
+#include <utility>
+
 #ifdef HAVE_CONFIG_H
 #include "sysconfig.h"
 #endif
 
-#include "wx/wxprec.h"
-#include "wx/object.h"
-#include "wx/config.h"
+#include <wx/wxprec.h>
+#include <wx/object.h>
+#include <wx/config.h>
 
 #ifdef __BORLANDC__
 	#pragma hdrstop
 #endif
 
 #ifndef WX_PRECOMP
-	#include "wx/wx.h"
+	#include <wx/wx.h>
 #endif
 
-#include "wx/dialog.h"
-#include "wx/notebook.h"
-#include "wx/checkbox.h"
-#include "wx/grid.h"
-#include "wx/wxhtml.h"
+#include <wx/dialog.h>
+#include <wx/notebook.h>
+#include <wx/checkbox.h>
+#include <wx/grid.h>
+#include <wx/wxhtml.h>
 #if defined(__APPLE__) || defined(_WIN32)
-#include "wx/filepicker.h"
+#include <wx/filepicker.h>
 #endif
 
 #include "tqslctrls.h"
 #include "wxutil.h"
 
-#include <map>
-#include <vector>
-
 using std::map;
 using std::pair;
 using std::vector;
@@ -95,9 +96,7 @@ enum {		// Window IDs
 	ID_PREF_FILE_DISPLAY_DUPES,
 	ID_PREF_FILE_LOG_TAB,
 	ID_PREF_FILE_CERTPWD,
-#if defined(__APPLE__)
 	ID_PREF_FILE_OLDCRYPTO,
-#endif
 	ID_PREF_FILE_LOGVFY,
 	ID_PREF_IGNORE_SECONDS,
 	ID_PREF_IGNORE_CALLSIGN,
@@ -145,9 +144,7 @@ class FilePrefs : public PrefsPanel {
  private:
 	wxTextCtrl *versions;
 	wxCheckBox *autobackup, *adifedit, *logtab, *certpwd;
-#if defined (__APPLE__)
 	wxCheckBox *oldcrypto;
-#endif
 #if !defined(__APPLE__) && !defined(_WIN32)
 	wxTextCtrl *dirPick;
 #else
