@@ -77,6 +77,7 @@ enum {		// Window IDs
 	ID_CRQ_PW1,
 	ID_CRQ_PW2,
 	ID_CRQ_SIGN,
+	ID_CRQ_SHOWALL,
 	ID_LCW_P12,
 	ID_LCW_TQ6,
 	ID_PREF_ROOT_CB,
@@ -85,7 +86,17 @@ enum {		// Window IDs
 	ID_PREF_ALLCERT_CB,
 	ID_CERT_OK_BUT,
 	ID_CERT_CAN_BUT,
-	ID_CERT_HELP_BUT
+	ID_CERT_HELP_BUT,
+	ID_LOC_CALL,
+	ID_LOC_DXCC,
+	ID_LOC_GRID,
+	ID_LOC_ITU,
+	ID_LOC_CQ,
+	ID_LOC_IOTA,
+	ID_LOC_PAS,
+	ID_LOC_SAS,
+	ID_LOC_PARK,
+	ID_LOC_ZERR
 };
 
 
