@@ -16,6 +16,7 @@
 
 #include <string>
 using std::string;
+#include <cstdio>
 
 #include "wxutil.h"
 #include "tqsltrace.h"
@@ -35,6 +36,7 @@ END_EVENT_TABLE()
 BEGIN_EVENT_TABLE(TQSLWizLocPage, TQSLWizPage)
 	EVT_CHECKBOX(-1, TQSLWizLocPage::OnCheckBoxEvent)
 	EVT_WIZARD_PAGE_CHANGING(wxID_ANY, TQSLWizLocPage::OnPageChanging)
+	EVT_WIZARD_CANCEL(-1, TQSLWizLocPage::OnCancel)
 	EVT_TEXT(ID_LOC_GRID, TQSLWizLocPage::OnTextEvent)
 	EVT_TEXT(ID_LOC_IOTA, TQSLWizLocPage::OnTextEvent)
 #if wxMAJOR_VERSION < 3 && (wxMAJOR_VERSION != 2 && wxMINOR_VERSION != 9)
@@ -138,6 +140,39 @@ TQSLWizard::GetPage(bool final) {
 	return _pages[page_num];
 }
 
+bool TQSLWizLocPage::GetChangedFields(wxString& changes) {
+	ForcedMap::iterator it;
+	for (it = forced.begin(); it != forced.end(); it++) {		// Something set
+		if (it->second != "") {
+			if (!changes.IsEmpty()) {
+				changes = changes + wxT(", ");
+			}
+			changes += wxString::FromUTF8(it->first.c_str());
+		}
+	}
+	for (it = userSet.begin(); it != userSet.end(); it++) {		// Something set
+		if (it->second != "") {
+			if (!changes.IsEmpty()) {
+				changes = changes + wxT(", ");
+			}
+			changes += wxString::FromUTF8(it->first.c_str());
+		}
+	}
+	return !changes.IsEmpty();
+}
+
+void TQSLWizLocPage::OnCancel(wxWizardEvent& ev) {
+	wxString changes(wxT(""));
+
+	if (GetChangedFields(changes)) {
+               if (wxMessageBox(wxString::Format(_("You have changed the following fields in this Station Location: %s\nDo you want to discard those changes?"), changes.c_str()),
+			_("Changes have not been saved"), wxYES_NO|wxICON_QUESTION, this) != wxYES) {
+			ev.Veto();
+		}
+	}
+}
+
+
 void TQSLWizLocPage::OnSize(wxSizeEvent& ev) {
 #if wxMAJOR_VERSION < 3 && (wxMAJOR_VERSION != 2 && wxMINOR_VERSION != 9)
 	TQSLWizPage::OnSize(ev);
@@ -233,6 +268,7 @@ TQSLWizLocPage::UpdateFields(int noupdate_field) {
 					tqsl_setLocationFieldCharData(loc, i, s.c_str());
 					 tx->ChangeValue(wxString::FromUTF8(s.c_str()));
 					forced[gabbi_name] = callsign;
+					gridFromDB = true;
 				}
 			}
 			if (strcmp(gabbi_name, "ITUZ") == 0) {
@@ -332,9 +368,6 @@ TQSLWizLocPage::UpdateFields(int noupdate_field) {
 				char buf[256];
 				tqsl_getLocationFieldCharData(loc, i, buf, sizeof buf);
 				tx->ChangeValue(wxString::FromUTF8(buf));
-				if (strcmp(gabbi_name, "GRIDSQUARE") == 0) {
-					gridFromDB = true;
-				}
 			}
 		} else if (in_type == TQSL_LOCATION_FIELD_BADZONE) {
 			int len;
@@ -681,7 +714,9 @@ TQSLWizLocPage::TQSLWizLocPage(TQSLWizard *_parent, tQSL_Location locp)
 	valMsg = wxT("");
 	invalidGrid = false;
 	allowBadGrid = false;
+	gridChanged = false;
 	gridFromDB = false;
+
 	tqsl_getStationLocationCapturePage(loc, &loc_page);
 	wxScreenDC sdc;
 	int label_w = 0;
@@ -994,57 +1029,49 @@ TQSLWizLocPage::validate() {
 			if (gridVal.size() == 0) {
 				continue;
 			}
+
 			string gridlist;
 			get_address_field(callsign, "grids", gridlist);
 			wxString editedGrids = wxT("");
+			bool oneGrid = false;
+			// Split the user-provided grid list (comma delimited set)
 			wxStringTokenizer grids(gridVal, wxT(","));	// Comma-separated list of squares
+			if (grids.CountTokens() == 1) {			// Not a list
+				oneGrid = true;
+			}
 			while (grids.HasMoreTokens()) {
 				wxString grid = grids.GetNextToken().Trim().Trim(false);
-				// Truncate to six character field
-				grid = grid.Left(6);
-				if (grid[0] <= 'z' && grid[0] >= 'a')
-					grid[0] = grid[0] - 'a' + 'A';	// Upper case first two
-				if (grid.size() > 1 && (grid[1] <= 'z' && grid[1] >= 'a'))
-					grid[1] = grid[1] - 'a' + 'A';
-				if (grid[0] < 'A' || grid[0] > 'R') {
-					if (valMsg.IsEmpty())
-						valMsg = wxString::Format(_("%s: Invalid Grid Square Field"), grid.c_str());
-				}
-				if (grid.size() > 1 && (grid[1] < 'A' || grid[1] > 'R')) {
-					if (valMsg.IsEmpty())
-						valMsg = wxString::Format(_("%s: Invalid Grid Square Field"), grid.c_str());
-				}
-				if (grid.size() > 2 && (grid[2] < '0' || grid[2] > '9')) {
-					if (valMsg.IsEmpty())
-						valMsg = wxString::Format(_("%s: Invalid Grid Square"), grid.c_str());
-				}
-				if (grid.size() < 4) {
-					if (valMsg.IsEmpty())
-						valMsg = wxString::Format(_("%s: Invalid Grid Square"), grid.c_str());
-				}
-				if (grid[3] < '0' || grid[3] > '9') {
-					if (valMsg.IsEmpty())
-						valMsg = wxString::Format(_("%s: Invalid Grid Square"), grid.c_str());
-				}
-
-				if (grid.size() > 4 && (grid[4] <= 'Z' && grid[4] >= 'A'))
-					grid[4] = grid[4] - 'A' + 'a';	// Lower case subsquare
-				if (grid.size() > 5 && (grid[5] <= 'Z' && grid[5] >= 'A'))
-					grid[5] = grid[5] - 'A' + 'a';
-
-				if (grid.size() > 4 && (grid[4] < 'a' || grid[4] > 'x')) {
-					if (valMsg.IsEmpty())
-						valMsg = wxString::Format(_("%s: Invalid Subsquare"), grid.c_str());
-				}
-				if (grid.size() > 5 && (grid[5] < 'a' || grid[5] > 'x')) {
-					if (valMsg.IsEmpty())
-						valMsg = wxString::Format(_("%s: Invalid Subsquare"), grid.c_str());
-				}
-				if (grid.size() != 6 && grid.size() != 4) {
-					// Not long enough yet or too long.
-					if (valMsg.IsEmpty())
-						valMsg = wxString::Format(_("%s: Invalid Grid Square"), grid.c_str());
+				// if only one grid, allow twelve. Else 6 characters
+				char newGrid[TQSL_GRID_MAX + 1];
+				int res = tqsl_verifyGridFormat(grid.ToUTF8(), oneGrid, newGrid, sizeof newGrid);
+				switch (res) {
+					case 0:				// No error
+						grid = wxString::FromUTF8(newGrid);
+						break;
+					case 1:				// API error
+						break;
+					case GRID_ERROR_INVALID_FIELD:
+						if (valMsg.IsEmpty())
+							valMsg = wxString::Format(_("%s: Invalid Grid Field"), grid.c_str());
+						break;
+					case GRID_ERROR_INVALID_SQUARE:
+						if (valMsg.IsEmpty())
+							valMsg = wxString::Format(_("%s: Invalid Grid Square"), grid.c_str());
+						break;
+					case GRID_ERROR_INVALID_SUBSQUARE:
+						if (valMsg.IsEmpty())
+							valMsg = wxString::Format(_("%s: Invalid Grid Subsquare"), grid.c_str());
+						break;
+					case GRID_ERROR_INVALID_SUBSUBSQUARE:
+						if (valMsg.IsEmpty())
+							valMsg = wxString::Format(_("%s: Invalid Grid Sub-subsquare"), grid.c_str());
+						break;
+					case GRID_ERROR_INVALID_FORMAT:
+						if (valMsg.IsEmpty())
+							valMsg = wxString::Format(_("%s: Invalid Gridsquare Format"), grid.c_str());
+						break;
 				}
+#ifdef USE_LOC_FOR_GRID
 				if (valMsg.IsEmpty() && !gridlist.empty() && !gridFromDB) {
 					string probe = string(grid.Left(4).mb_str());
 					if (gridlist.find(probe) == string::npos) {
@@ -1054,6 +1081,47 @@ TQSLWizLocPage::validate() {
 						invalidGrid = false;
 					}
 				}
+#else
+				if (valMsg.IsEmpty() && !gridFromDB) {
+					int cur_page;
+					tqsl_getStationLocationCapturePage(loc, &cur_page);
+					tqsl_setStationLocationCapturePage(loc, second_page);
+					int dxcc = -1;
+					if (!tqsl_getLocationDXCCEntity(loc, &dxcc)) {
+						int numf;
+						char gname[50];
+						char pas[50];
+						char label[50];
+						pas[0] = '\0';
+						tqsl_getNumLocationField(loc, &numf);
+						for (int i = 0; i < numf; i++) {
+							if (!tqsl_getLocationFieldDataGABBI(loc, i, gname, sizeof gname)) {
+								if (isPAS(gname)) {
+									tqsl_getLocationFieldCharData(loc, i, pas, sizeof pas);
+									tqsl_getLocationFieldDataLabel(loc, i, label, sizeof label);
+									break;
+								}
+							}
+						}
+						wxString userGrid = grid.Upper().Left(4);
+						int goodGrid = 0;
+						if (!tqsl_validateVUCCGrid(dxcc, pas, userGrid.ToUTF8(), &goodGrid)) {
+							if ((goodGrid & TQSL_VALID_VUCC_ENT) && (goodGrid & TQSL_VALID_VUCC_PAS)) {
+								invalidGrid = false;
+							} else if (!(goodGrid & TQSL_VALID_VUCC_ENT)) {
+								valMsg = wxString::Format(_("Grid %s is not correct for your DXCC Entity. Click 'Next' again to use it anyway."), grid.c_str());
+								invalidGrid = true;
+							} else {
+								valMsg = wxString::Format(_("Grid %s is not correct for your %s. Click 'Next' again to use it anyway."), grid.c_str(), label);
+								invalidGrid = true;
+							}
+						} else {
+							invalidGrid = false;	// Check failed, all OK
+						}
+					}
+					tqsl_setStationLocationCapturePage(loc, cur_page);
+				}
+#endif
 				if (!editedGrids.IsEmpty())
 					editedGrids += wxT(",");
 				editedGrids += grid;
@@ -1151,7 +1219,15 @@ TQSLWizLocPage::TransferDataFromWindow() {
 				case TQSL_LOCATION_FIELD_LIST:
 					break;
 				case TQSL_LOCATION_FIELD_TEXT:
+					char lastValue[100], newValue[100];
+					tqsl_getLocationFieldCharData(loc, i, lastValue, sizeof lastValue);
 					tqsl_setLocationFieldCharData(loc, i, (reinterpret_cast<wxTextCtrl *>(p1_controls[i]))->GetValue().ToUTF8());
+					char gabbi_name[40];
+					tqsl_getLocationFieldDataGABBI(loc, i, gabbi_name, sizeof gabbi_name);
+					if (!strcmp(gabbi_name, "GRIDSQUARE")) {
+						tqsl_getLocationFieldCharData(loc, i, newValue, sizeof newValue);
+						gridChanged = (strcmp(lastValue, newValue) != 0);
+					}
 					break;
 			}
 		}
@@ -1178,11 +1254,14 @@ TQSLWizLocPage::OnPageChanging(wxWizardEvent& ev) {
 
 	validate();
 	if (valMsg.Len() > 0 && ev.GetDirection()) {
-		if (!allowBadGrid) {
+		if (!allowBadGrid || gridChanged) {
 			ev.Veto();
-			allowBadGrid = true;		// Don't allow going forward once
+			if (gridChanged) {
+				allowBadGrid = true;		// Don't allow going forward once
+			}
 		} else {
 			allowBadGrid = false;
+			gridChanged = false;
 		}
 		if (!invalidGrid) {
 			wxMessageBox(valMsg, _("Error"), wxOK | wxICON_ERROR, this);
@@ -1195,8 +1274,22 @@ BEGIN_EVENT_TABLE(TQSLWizFinalPage, TQSLWizPage)
 	EVT_LISTBOX_DCLICK(TQSL_ID_LOW, TQSLWizFinalPage::OnListbox)
 	EVT_TEXT(TQSL_ID_LOW+1, TQSLWizFinalPage::check_valid)
 	EVT_WIZARD_PAGE_CHANGING(wxID_ANY, TQSLWizFinalPage::OnPageChanging)
+	EVT_WIZARD_CANCEL(-1, TQSLWizFinalPage::OnCancel)
 END_EVENT_TABLE()
 
+void
+TQSLWizFinalPage::OnCancel(wxWizardEvent& ev) {
+	TQSLWizLocPage* first = static_cast<TQSLWizLocPage*>(GetParent()->GetPage(0));
+	wxString changes(wxT(""));
+
+	if (first->GetChangedFields(changes)) {
+               if (wxMessageBox(wxString::Format(_("You have changed the following fields in this Station Location: %s\nDo you want to discard those changes?"), changes.c_str()),
+			_("Changes have not been saved"), wxYES_NO|wxICON_QUESTION, this) != wxYES) {
+			ev.Veto();
+		}
+	}
+}
+
 void
 TQSLWizFinalPage::OnListbox(wxCommandEvent &) {
 	tqslTrace("TQSLWizFinalPage::OnListbox", NULL);
