@@ -11,23 +11,30 @@
 #ifndef __tqslwiz_h
 #define __tqslwiz_h
 
+#include <cstdio>
+#include <string>
+
 #ifdef HAVE_CONFIG_H
 #include "sysconfig.h"
 #endif
 
-#include "wx/wxprec.h"
+#include <wx/wxprec.h>
 
 #ifdef __BORLANDC__
 	#pragma hdrstop
 #endif
 
 #ifndef WX_PRECOMP
-	#include "wx/wx.h"
+	#include <wx/wx.h>
 #endif
 
+#include <wx/radiobox.h>
+#include <wx/odcombo.h>
+
+#include <vector>
+#include <map>
+
 #include "extwizard.h"
-#include "wx/radiobox.h"
-#include "wx/odcombo.h"
 
 #include "certtree.h"
 
@@ -36,9 +43,6 @@
 #endif
 #include "tqsllib.h"
 
-#include <vector>
-#include <map>
-
 using std::map;
 using std::vector;
 
@@ -95,6 +99,7 @@ class TQSLWizLocPage : public TQSLWizPage {
 	int loc_page;
 	bool invalidGrid;
 	bool allowBadGrid;
+	bool gridChanged;
 	bool gridFromDB;
 	wxBoxSizer* sizer;
 	void UpdateFields(int noupdate_field = -1);
@@ -103,6 +108,8 @@ class TQSLWizLocPage : public TQSLWizPage {
 	virtual TQSLWizPage *GetNext() const;
 	void OnSize(wxSizeEvent&);
 	void OnPageChanging(wxWizardEvent &);
+	void OnCancel(wxWizardEvent &);
+	bool GetChangedFields(wxString &);
 
  private:
 	vector<void *> p1_controls;	// First page
@@ -158,6 +165,7 @@ class TQSLWizFinalPage : public TQSLWizPage {
 	void OnListbox(wxCommandEvent &);
 	virtual const char *validate();
 	void OnPageChanging(wxWizardEvent &);
+	void OnCancel(wxWizardEvent &);
  private:
 	wxListBox *namelist;
 	wxTextCtrl *newname;
