@@ -19,9 +19,9 @@
 #include <wx/filename.h>
 #include <wx/treectrl.h>
 #include <wx/textctrl.h>
+#include <vector>
 #include "tqsllib.h"
 #include "tqslerrno.h"
-#include <vector>
 
 #if wxMAJOR_VERSION == 3 && wxMINOR_VERSION > 0
 #define WX31		// wxWidgets isn't done until stuff doesn't run
@@ -89,6 +89,44 @@ urlEncode(wxString& str) {
 	return str;
 }
 
+// Encode password for url
+wxString
+passwordEncode(wxString& pw) {
+	pw.Replace(wxT(" "), wxT("%20"), true);
+	pw.Replace(wxT("!"), wxT("%21"), true);
+	pw.Replace(wxT("\""), wxT("%22"), true);
+	pw.Replace(wxT("#"), wxT("%23"), true);
+	pw.Replace(wxT("$"), wxT("%24"), true);
+	pw.Replace(wxT("%"), wxT("%25"), true);
+	pw.Replace(wxT("&"), wxT("%26"), true);
+	pw.Replace(wxT("'"), wxT("%27"), true);
+	pw.Replace(wxT("("), wxT("%28"), true);
+	pw.Replace(wxT(")"), wxT("%2A"), true);
+	pw.Replace(wxT("+"), wxT("%2B"), true);
+	pw.Replace(wxT(","), wxT("%2C"), true);
+	pw.Replace(wxT("-"), wxT("%2D"), true);
+	pw.Replace(wxT("."), wxT("%2E"), true);
+	pw.Replace(wxT("/"), wxT("%2F"), true);
+	pw.Replace(wxT(":"), wxT("%3A"), true);
+	pw.Replace(wxT(";"), wxT("%3B"), true);
+	pw.Replace(wxT("<"), wxT("%3C"), true);
+	pw.Replace(wxT("="), wxT("%3D"), true);
+	pw.Replace(wxT(">"), wxT("%3E"), true);
+	pw.Replace(wxT("?"), wxT("%3F"), true);
+	pw.Replace(wxT("@"), wxT("%40"), true);
+	pw.Replace(wxT("["), wxT("%5B"), true);
+	pw.Replace(wxT("\\"), wxT("%5C"), true);
+	pw.Replace(wxT("]"), wxT("%5D"), true);
+	pw.Replace(wxT("^"), wxT("%5E"), true);
+	pw.Replace(wxT("_"), wxT("%5F"), true);
+	pw.Replace(wxT("."), wxT("%60"), true);
+	pw.Replace(wxT("{"), wxT("%7B"), true);
+	pw.Replace(wxT("|"), wxT("%7C"), true);
+	pw.Replace(wxT("}"), wxT("%7D"), true);
+	pw.Replace(wxT("~"), wxT("%7E"), true);
+	return pw;
+}
+
 // Convert UTF-8 string to UCS-2 (MS Unicode default)
 int
 utf8_to_ucs2(const char *in, char *out, size_t buflen) {
@@ -157,7 +195,7 @@ static const char *error_strings[] = {
 	__("PKCS#12 file not TQSL compatible"),			/* TQSL_PKCS12_ERROR */
 	__("Callsign Certificate not TQSL compatible"),		/* TQSL_CERT_TYPE_ERROR */
 	__("Date out of range"),				/* TQSL_DATE_OUT_OF_RANGE */
-	__("Already Uploaded QSO detected"),			/* TQSL_DUPLICATE_QSO */
+	__("Previously Signed QSO detected"),			/* TQSL_DUPLICATE_QSO */
 	__("Database error"),					/* TQSL_DB_ERROR */
 	__("The selected station location could not be found"),	/* TQSL_LOCATION_NOT_FOUND */
 	__("The selected callsign could not be found"),		/* TQSL_CALL_NOT_FOUND */
@@ -167,10 +205,17 @@ static const char *error_strings[] = {
 	__("Callsign certificate could not be installed"),	/* TQSL_CERT_ERROR */
 	__("Callsign Certificate does not match QSO details"),	/* TQSL_CERT_MISMATCH */
 	__("Station Location does not match QSO details"),	/* TQSL_LOCATION_MISMATCH */
+	__("New DB"),						/* TQSL_NEW_UPLOAD_DB */
+	__("Gridsquare is inconsistent with Station Location"),	/* TQSL_INCONSISTENT_GRID */
+	__("ADIF field has invalid contents"),			/* TQSL_INVALID_ADIF */
 	__("This Callsign Certificate cannot be installed as the first date where it is valid is in the future. Check if your computer is set to the proper date.\n\n"),
-        __("This Callsign Certificate cannot be installed as it has expired. Check if your computer is set to the proper date and that this is the latest Callsign Certificate.\n\n"),
+        __("This Callsign Certificate cannot be installed as it has expired. Check if your computer is set to the proper date and that this is the latest Callsign Certificate.\n\n")
 };
 
+#ifdef _WIN32
+#define strtok_r strtok_s
+#endif
+
 static wxString
 getLocalizedErrorString_v(int err) {
 	int adjusted_err;
@@ -258,12 +303,12 @@ getLocalizedErrorString_v(int err) {
 		static_cast<int>(sizeof error_strings / sizeof error_strings[0])) {
 		return wxString::Format(_("Invalid error code: %d"), err);
 	}
-
 	if (err == TQSL_CERT_MISMATCH || err == TQSL_LOCATION_MISMATCH) {
 		const char *fld, *cert, *qso;
-		fld = strtok(tQSL_CustomError, "|");
-		cert = strtok(NULL, "|");
-		qso = strtok(NULL, "|");
+		char *state = NULL;
+		fld = strtok_r(tQSL_CustomError, "|", &state);
+		cert = strtok_r(NULL, "|", &state);
+		qso = strtok_r(NULL, "|", &state);
 		if (qso == NULL) {		// Nothing in the cert
 			qso = cert;
 			cert = "none";
@@ -276,23 +321,33 @@ getLocalizedErrorString_v(int err) {
 		composed = composed + wxT("\n") + wxString::Format(_("The %s '%hs' has value '%hs' while QSO has '%hs'"), tp.c_str(), fld, cert, qso);
 		return composed;
 	}
-	if (err == (TQSL_LOCATION_MISMATCH | 0x1000)) {
+	if (err == (TQSL_LOCATION_MISMATCH | TQSL_MSG_FLAGGED)) {
 		const char *fld, *val;
-		fld = strtok(tQSL_CustomError, "|");
-		val = strtok(NULL, "|");
+		char *state = NULL;
+		fld = strtok_r(tQSL_CustomError, "|", &state);
+		val = strtok_r(NULL, "|", &state);
 	 	wxString composed(_("This log has invalid QSO information"));
 		// TRANSLATORS: This message is for QSO details. For example, 'The log being signed has 'US County' set to Foobar which is not valid'
 		composed = composed + wxT("\n") + wxString::Format(_("The log being signed has '%hs' set to value '%hs' which is not valid"), fld, val);
 		return composed;
 	}
-	if (err == (TQSL_CERT_NOT_FOUND | 0x1000)) {
+	if (err == (TQSL_CERT_NOT_FOUND | TQSL_MSG_FLAGGED)) {
 		err = TQSL_CERT_NOT_FOUND;
 		const char *call, *ent;
-		call = strtok(tQSL_CustomError, "|");
-		ent = strtok(NULL, "|");
+		char *state = NULL;
+		call = strtok_r(tQSL_CustomError, "|", &state);
+		ent = strtok_r(NULL, "|", &state);
 		wxString composed = wxString::Format(_("There is no valid callsign certificate for %hs in entity %hs available. This QSO cannot be signed"), call, ent);
 		return composed;
 	}
+	if (err == TQSL_INVALID_ADIF) {
+		const char *fname, *contents;
+		char *state = NULL;
+		fname = strtok_r(tQSL_CustomError, "|", &state);
+		contents = strtok_r(NULL, "|", &state);
+		wxString composed = wxString::Format(_("ADIF content is not valid. '%hs' is not valid for %hs"), contents, fname);
+		return composed;
+	}
 	return wxGetTranslation(wxString::FromUTF8(error_strings[adjusted_err]));
 }
 
@@ -309,272 +364,48 @@ getLocalizedErrorString() {
  * wxWidgets used to keep language IDs stable, by adding to the end of the
  * enum. wx3 seems to have broken that promise by injecting several languages
  * in the middle. But we use language IDs to specify the local language. Horrid.
- * So, here's a map from old to new.
+ * So, map from old to new. Unfortunately, these aren't stable so adding new
+ * languages means editing this.
  */
 
-#if wxMAJOR_VERSION > 2
-	struct langMap {
-		wxLanguage wx3;
-		int wx2;
-	};
-
-	struct langMap mapping[] = {
-	    { wxLANGUAGE_DEFAULT, 0 },
-	    { wxLANGUAGE_UNKNOWN, 1 },
-	    { wxLANGUAGE_ABKHAZIAN, 2 },
-	    { wxLANGUAGE_AFAR, 3 },
-	    { wxLANGUAGE_AFRIKAANS, 4 },
-	    { wxLANGUAGE_ALBANIAN, 5 },
-	    { wxLANGUAGE_AMHARIC, 6 },
-	    { wxLANGUAGE_ARABIC, 7 },
-	    { wxLANGUAGE_ARABIC_ALGERIA, 8 },
-	    { wxLANGUAGE_ARABIC_BAHRAIN, 9 },
-	    { wxLANGUAGE_ARABIC_EGYPT, 10 },
-	    { wxLANGUAGE_ARABIC_IRAQ, 11 },
-	    { wxLANGUAGE_ARABIC_JORDAN, 12 },
-	    { wxLANGUAGE_ARABIC_KUWAIT, 13 },
-	    { wxLANGUAGE_ARABIC_LEBANON, 14 },
-	    { wxLANGUAGE_ARABIC_LIBYA, 15 },
-	    { wxLANGUAGE_ARABIC_MOROCCO, 16 },
-	    { wxLANGUAGE_ARABIC_OMAN, 17 },
-	    { wxLANGUAGE_ARABIC_QATAR, 18 },
-	    { wxLANGUAGE_ARABIC_SAUDI_ARABIA, 19 },
-	    { wxLANGUAGE_ARABIC_SUDAN, 20 },
-	    { wxLANGUAGE_ARABIC_SYRIA, 21 },
-	    { wxLANGUAGE_ARABIC_TUNISIA, 22 },
-	    { wxLANGUAGE_ARABIC_UAE, 23 },
-	    { wxLANGUAGE_ARABIC_YEMEN, 24 },
-	    { wxLANGUAGE_ARMENIAN, 25 },
-	    { wxLANGUAGE_ASSAMESE, 26 },
-	    { wxLANGUAGE_ASTURIAN, wxLANGUAGE_ASTURIAN + 0x8000 },
-	    { wxLANGUAGE_AYMARA, 27 },
-	    { wxLANGUAGE_AZERI, 28 },
-	    { wxLANGUAGE_AZERI_CYRILLIC, 29 },
-	    { wxLANGUAGE_AZERI_LATIN, 30 },
-	    { wxLANGUAGE_BASHKIR, 31 },
-	    { wxLANGUAGE_BASQUE, 32 },
-	    { wxLANGUAGE_BELARUSIAN, 33 },
-	    { wxLANGUAGE_BENGALI, 34 },
-	    { wxLANGUAGE_BHUTANI, 35 },
-	    { wxLANGUAGE_BIHARI, 36 },
-	    { wxLANGUAGE_BISLAMA, 37 },
-	    { wxLANGUAGE_BOSNIAN, wxLANGUAGE_BOSNIAN + 0x8000 },
-	    { wxLANGUAGE_BRETON, 38 },
-	    { wxLANGUAGE_BULGARIAN, 39 },
-	    { wxLANGUAGE_BURMESE, 40 },
-#ifndef WX31
-	    { wxLANGUAGE_CAMBODIAN, 41 },
-#endif
-	    { wxLANGUAGE_CATALAN, 42 },
-	    { wxLANGUAGE_CHINESE, 43 },
-	    { wxLANGUAGE_CHINESE_SIMPLIFIED, 44 },
-	    { wxLANGUAGE_CHINESE_TRADITIONAL, 45 },
-	    { wxLANGUAGE_CHINESE_HONGKONG, 46 },
-	    { wxLANGUAGE_CHINESE_MACAU, 47 },
-	    { wxLANGUAGE_CHINESE_SINGAPORE, 48 },
-	    { wxLANGUAGE_CHINESE_TAIWAN, 49 },
-	    { wxLANGUAGE_CORSICAN, 50 },
-	    { wxLANGUAGE_CROATIAN, 51 },
-	    { wxLANGUAGE_CZECH, 52 },
-	    { wxLANGUAGE_DANISH, 53 },
-	    { wxLANGUAGE_DUTCH, 54 },
-	    { wxLANGUAGE_DUTCH_BELGIAN, 55 },
-	    { wxLANGUAGE_ENGLISH, 56 },
-	    { wxLANGUAGE_ENGLISH_UK, 57 },
-	    { wxLANGUAGE_ENGLISH_US, 58 },
-	    { wxLANGUAGE_ENGLISH_AUSTRALIA, 59 },
-	    { wxLANGUAGE_ENGLISH_BELIZE, 60 },
-	    { wxLANGUAGE_ENGLISH_BOTSWANA, 61 },
-	    { wxLANGUAGE_ENGLISH_CANADA, 62 },
-	    { wxLANGUAGE_ENGLISH_CARIBBEAN, 63 },
-	    { wxLANGUAGE_ENGLISH_DENMARK, 64 },
-	    { wxLANGUAGE_ENGLISH_EIRE, 65 },
-	    { wxLANGUAGE_ENGLISH_JAMAICA, 66 },
-	    { wxLANGUAGE_ENGLISH_NEW_ZEALAND, 67 },
-	    { wxLANGUAGE_ENGLISH_PHILIPPINES, 68 },
-	    { wxLANGUAGE_ENGLISH_SOUTH_AFRICA, 69 },
-	    { wxLANGUAGE_ENGLISH_TRINIDAD, 70 },
-	    { wxLANGUAGE_ENGLISH_ZIMBABWE, 71 },
-	    { wxLANGUAGE_ESPERANTO, 72 },
-	    { wxLANGUAGE_ESTONIAN, 73 },
-	    { wxLANGUAGE_FAEROESE, 74 },
-	    { wxLANGUAGE_FARSI, 75 },
-	    { wxLANGUAGE_FIJI, 76 },
-	    { wxLANGUAGE_FINNISH, 77 },
-	    { wxLANGUAGE_FRENCH, 78 },
-	    { wxLANGUAGE_FRENCH_BELGIAN, 79 },
-	    { wxLANGUAGE_FRENCH_CANADIAN, 80 },
-	    { wxLANGUAGE_FRENCH_LUXEMBOURG, 81 },
-	    { wxLANGUAGE_FRENCH_MONACO, 82 },
-	    { wxLANGUAGE_FRENCH_SWISS, 83 },
-	    { wxLANGUAGE_FRISIAN, 84 },
-	    { wxLANGUAGE_GALICIAN, 85 },
-	    { wxLANGUAGE_GEORGIAN, 86 },
-	    { wxLANGUAGE_GERMAN, 87 },
-	    { wxLANGUAGE_GERMAN_AUSTRIAN, 88 },
-	    { wxLANGUAGE_GERMAN_BELGIUM, 89 },
-	    { wxLANGUAGE_GERMAN_LIECHTENSTEIN, 90 },
-	    { wxLANGUAGE_GERMAN_LUXEMBOURG, 91 },
-	    { wxLANGUAGE_GERMAN_SWISS, 92 },
-	    { wxLANGUAGE_GREEK, 93 },
-	    { wxLANGUAGE_GREENLANDIC, 94 },
-	    { wxLANGUAGE_GUARANI, 95 },
-	    { wxLANGUAGE_GUJARATI, 96 },
-	    { wxLANGUAGE_HAUSA, 97 },
-	    { wxLANGUAGE_HEBREW, 98 },
-	    { wxLANGUAGE_HINDI, 99 },
-	    { wxLANGUAGE_HUNGARIAN, 100 },
-	    { wxLANGUAGE_ICELANDIC, 101 },
-	    { wxLANGUAGE_INDONESIAN, 102 },
-	    { wxLANGUAGE_INTERLINGUA, 103 },
-	    { wxLANGUAGE_INTERLINGUE, 104 },
-	    { wxLANGUAGE_INUKTITUT, 105 },
-	    { wxLANGUAGE_INUPIAK, 106 },
-	    { wxLANGUAGE_IRISH, 107 },
-	    { wxLANGUAGE_ITALIAN, 108 },
-	    { wxLANGUAGE_ITALIAN_SWISS, 109 },
-	    { wxLANGUAGE_JAPANESE, 110 },
-	    { wxLANGUAGE_JAVANESE, 111 },
-#ifdef WX31
-	    { wxLANGUAGE_KABYLE, wxLANGUAGE_KABYLE + 0x8000 },
-#endif
-	    { wxLANGUAGE_KANNADA, 112 },
-	    { wxLANGUAGE_KASHMIRI, 113 },
-	    { wxLANGUAGE_KASHMIRI_INDIA, 114 },
-	    { wxLANGUAGE_KAZAKH, 115 },
-	    { wxLANGUAGE_KERNEWEK, 116 },
-#ifdef WX31
-	    { wxLANGUAGE_KHMER, 41 },
-#endif
-	    { wxLANGUAGE_KINYARWANDA, 117 },
-	    { wxLANGUAGE_KIRGHIZ, 118 },
-	    { wxLANGUAGE_KIRUNDI, 119 },
-	    { wxLANGUAGE_KONKANI, 120 },
-	    { wxLANGUAGE_KOREAN, 121 },
-	    { wxLANGUAGE_KURDISH, 122 },
-	    { wxLANGUAGE_LAOTHIAN, 123 },
-	    { wxLANGUAGE_LATIN, 124 },
-	    { wxLANGUAGE_LATVIAN, 125 },
-	    { wxLANGUAGE_LINGALA, 126 },
-	    { wxLANGUAGE_LITHUANIAN, 127 },
-	    { wxLANGUAGE_MACEDONIAN, 128 },
-	    { wxLANGUAGE_MALAGASY, 129 },
-	    { wxLANGUAGE_MALAY, 130 },
-	    { wxLANGUAGE_MALAYALAM, 131 },
-	    { wxLANGUAGE_MALAY_BRUNEI_DARUSSALAM, 132 },
-	    { wxLANGUAGE_MALAY_MALAYSIA, 133 },
-	    { wxLANGUAGE_MALTESE, 134 },
-	    { wxLANGUAGE_MANIPURI, 135 },
-	    { wxLANGUAGE_MAORI, 136 },
-	    { wxLANGUAGE_MARATHI, 137 },
-	    { wxLANGUAGE_MOLDAVIAN, 138 },
-	    { wxLANGUAGE_MONGOLIAN, 139 },
-	    { wxLANGUAGE_NAURU, 140 },
-	    { wxLANGUAGE_NEPALI, 141 },
-	    { wxLANGUAGE_NEPALI_INDIA, 142 },
-	    { wxLANGUAGE_NORWEGIAN_BOKMAL, 143 },
-	    { wxLANGUAGE_NORWEGIAN_NYNORSK, 144 },
-	    { wxLANGUAGE_OCCITAN, 145 },
-	    { wxLANGUAGE_ORIYA, 146 },
-	    { wxLANGUAGE_OROMO, 147 },
-	    { wxLANGUAGE_PASHTO, 148 },
-	    { wxLANGUAGE_POLISH, 149 },
-	    { wxLANGUAGE_PORTUGUESE, 150 },
-	    { wxLANGUAGE_PORTUGUESE_BRAZILIAN, 151 },
-	    { wxLANGUAGE_PUNJABI, 152 },
-	    { wxLANGUAGE_QUECHUA, 153 },
-	    { wxLANGUAGE_RHAETO_ROMANCE, 154 },
-	    { wxLANGUAGE_ROMANIAN, 155 },
-	    { wxLANGUAGE_RUSSIAN, 156 },
-	    { wxLANGUAGE_RUSSIAN_UKRAINE, 157 },
-	    { wxLANGUAGE_SAMI, 0x20000000 },
-	    { wxLANGUAGE_SAMOAN, 158 },
-	    { wxLANGUAGE_SANGHO, 159 },
-	    { wxLANGUAGE_SANSKRIT, 160 },
-	    { wxLANGUAGE_SCOTS_GAELIC, 161 },
-	    { wxLANGUAGE_SERBIAN, 162 },
-	    { wxLANGUAGE_SERBIAN_CYRILLIC, 163 },
-	    { wxLANGUAGE_SERBIAN_LATIN, 164 },
-	    { wxLANGUAGE_SERBO_CROATIAN, 165 },
-	    { wxLANGUAGE_SESOTHO, 166 },
-	    { wxLANGUAGE_SETSWANA, 167 },
-	    { wxLANGUAGE_SHONA, 168 },
-	    { wxLANGUAGE_SINDHI, 169 },
-	    { wxLANGUAGE_SINHALESE, 170 },
-	    { wxLANGUAGE_SISWATI, 171 },
-	    { wxLANGUAGE_SLOVAK, 172 },
-	    { wxLANGUAGE_SLOVENIAN, 173 },
-	    { wxLANGUAGE_SOMALI, 174 },
-	    { wxLANGUAGE_SPANISH, 175 },
-	    { wxLANGUAGE_SPANISH_ARGENTINA, 176 },
-	    { wxLANGUAGE_SPANISH_BOLIVIA, 177 },
-	    { wxLANGUAGE_SPANISH_CHILE, 178 },
-	    { wxLANGUAGE_SPANISH_COLOMBIA, 179 },
-	    { wxLANGUAGE_SPANISH_COSTA_RICA, 180 },
-	    { wxLANGUAGE_SPANISH_DOMINICAN_REPUBLIC, 181 },
-	    { wxLANGUAGE_SPANISH_ECUADOR, 182 },
-	    { wxLANGUAGE_SPANISH_EL_SALVADOR, 183 },
-	    { wxLANGUAGE_SPANISH_GUATEMALA, 184 },
-	    { wxLANGUAGE_SPANISH_HONDURAS, 185 },
-	    { wxLANGUAGE_SPANISH_MEXICAN, 186 },
-	    { wxLANGUAGE_SPANISH_MODERN, 187 },
-	    { wxLANGUAGE_SPANISH_NICARAGUA, 188 },
-	    { wxLANGUAGE_SPANISH_PANAMA, 189 },
-	    { wxLANGUAGE_SPANISH_PARAGUAY, 190 },
-	    { wxLANGUAGE_SPANISH_PERU, 191 },
-	    { wxLANGUAGE_SPANISH_PUERTO_RICO, 192 },
-	    { wxLANGUAGE_SPANISH_URUGUAY, 193 },
-	    { wxLANGUAGE_SPANISH_US, 194 },
-	    { wxLANGUAGE_SPANISH_VENEZUELA, 195 },
-	    { wxLANGUAGE_SUNDANESE, 196 },
-	    { wxLANGUAGE_SWAHILI, 197 },
-	    { wxLANGUAGE_SWEDISH, 198 },
-	    { wxLANGUAGE_SWEDISH_FINLAND, 199 },
-	    { wxLANGUAGE_TAGALOG, 200 },
-	    { wxLANGUAGE_TAJIK, 201 },
-	    { wxLANGUAGE_TAMIL, 202 },
-	    { wxLANGUAGE_TATAR, 203 },
-	    { wxLANGUAGE_TELUGU, 204 },
-	    { wxLANGUAGE_THAI, 205 },
-	    { wxLANGUAGE_TIBETAN, 206 },
-	    { wxLANGUAGE_TIGRINYA, 207 },
-	    { wxLANGUAGE_TONGA, 208 },
-	    { wxLANGUAGE_TSONGA, 209 },
-	    { wxLANGUAGE_TURKISH, 210 },
-	    { wxLANGUAGE_TURKMEN, 211 },
-	    { wxLANGUAGE_TWI, 212 },
-	    { wxLANGUAGE_UIGHUR, 213 },
-	    { wxLANGUAGE_UKRAINIAN, 214 },
-	    { wxLANGUAGE_URDU, 215 },
-	    { wxLANGUAGE_URDU_INDIA, 216 },
-	    { wxLANGUAGE_URDU_PAKISTAN, 217 },
-	    { wxLANGUAGE_UZBEK, 218 },
-	    { wxLANGUAGE_UZBEK_CYRILLIC, 219 },
-	    { wxLANGUAGE_UZBEK_LATIN, 220 },
-	    { wxLANGUAGE_VALENCIAN, 0x1fffffff },
-	    { wxLANGUAGE_VIETNAMESE, 221 },
-	    { wxLANGUAGE_VOLAPUK, 222 },
-	    { wxLANGUAGE_WELSH, 223 },
-	    { wxLANGUAGE_WOLOF, 224 },
-	    { wxLANGUAGE_XHOSA, 225 },
-	    { wxLANGUAGE_YIDDISH, 226 },
-	    { wxLANGUAGE_YORUBA, 227 },
-	    { wxLANGUAGE_ZHUANG, 228 },
-	    { wxLANGUAGE_ZULU, 229 },
-#ifndef WX31
-	    { wxLANGUAGE_KABYLE, wxLANGUAGE_KABYLE + 0x8000 },	// wx3.1 moved this to after Javanese.
-#endif
-	    { wxLANGUAGE_USER_DEFINED, 230 },
-	    { wxLANGUAGE_DEFAULT, -1}
-	};
-
-#endif // WX3
-
 wxLanguage langWX2toWX3(wxLanguage wx2) {
 #if wxMAJOR_VERSION > 2
-	for (unsigned int i = 0; i < WXSIZEOF(mapping); i++) {
-		if (mapping[i].wx2 == -1) return wx2;
-		if (mapping[i].wx2 == wx2) return mapping[i].wx3;
+	switch (wx2) {
+		case 42:
+			return wxLANGUAGE_CATALAN;
+		case 43: // Map wxLANGUAGE_CHINESE to SIMPLIFIED
+		case 44:
+			return wxLANGUAGE_CHINESE_SIMPLIFIED;
+		case 45:
+			return wxLANGUAGE_CHINESE_TRADITIONAL;
+		case 56:
+			return wxLANGUAGE_ENGLISH;
+		case 77:
+			return wxLANGUAGE_FINNISH;
+		case 78:
+			return wxLANGUAGE_FRENCH;
+		case 87:
+			return wxLANGUAGE_GERMAN;
+		case 99:
+			return wxLANGUAGE_HINDI;
+		case 108:
+			return wxLANGUAGE_ITALIAN;
+		case 110:
+			return wxLANGUAGE_JAPANESE;
+		case 149:
+			return wxLANGUAGE_POLISH;
+		case 150:
+			return wxLANGUAGE_PORTUGUESE;
+		case 156:
+			return wxLANGUAGE_RUSSIAN;
+		case 175:
+			return wxLANGUAGE_SPANISH;
+		case 198:
+			return wxLANGUAGE_SWEDISH;
+		case 210:
+			return wxLANGUAGE_TURKISH;
+		default:
+			return wx2;
 	}
 #endif
 	return wx2;
@@ -729,7 +560,7 @@ wxAccStatus TreeCtrlAx::GetDefaultAction(int WXUNUSED(childId), wxString* action
 // Returns the description for this object or a child.
 wxAccStatus TreeCtrlAx::GetDescription(int childId, wxString *description) {
 	if (childId == wxACC_SELF) {
-		*description = _("Tree Ctrl - use control/option/arrow keys to navigate");
+		*description = wxT("Tree Ctrl - use control/option/arrow keys to navigate");
 	} else {
 		description->Clear();
 	}
@@ -946,7 +777,7 @@ wxAccStatus ComboBoxAx::GetDescription(int childId, wxString *description) {
 		return wxACC_FAIL;
 
 	if (childId == wxACC_SELF) {
-		*description = _("ComboBox - Use the arrow keys to navigate the list. Typing the first letter of an entry will navigate to that entry");
+		*description = wxT("ComboBox - Use the arrow keys to navigate the list. Typing the first letter of an entry will navigate to that entry");
 	} else {
 		*description = ctrl->GetValue();
 	}
