@@ -7,13 +7,16 @@
     revision             : $Id$
  ***************************************************************************/
 
-#define TQSLLIB_DEF
+//#define TQSLLIB_DEF
+
+#include "adif.h"
 
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <cstdio>
 #ifdef HAVE_CONFIG_H
 #include "sysconfig.h"
 #endif
@@ -106,7 +109,7 @@ tqsl_beginADIF(tQSL_ADIF *adifp, const char *filename) {
 	}
 	*((struct TQSL_ADIF **)adifp) = adif;
 	return 0;
- err:
+err:
 	free_adif(adif);
 	return 1;
 }
