@@ -11,24 +11,26 @@
 
 #define TQSLLIB_DEF
 
-#include "tqsllib.h"
-
 #include "tqslconvert.h"
+
 #include <stdio.h>
 #include <errno.h>
 #include <sys/stat.h>
 #include <signal.h>
-#include "tqslerrno.h"
+#include <ctype.h>
+#include <locale.h>
+#include <sqlite3.h>
 #include <cstring>
 #include <string>
 #include <vector>
 #include <map>
 #include <algorithm>
-#include <ctype.h>
 #include <set>
-#include <sqlite3.h>
+#include <cstdio>
+
+#include "tqslerrno.h"
+#include "tqsllib.h"
 
-#include <locale.h>
 //#include <iostream>
 
 #ifndef _WIN32
@@ -91,6 +93,7 @@ class TQSL_CONVERTER {
 	bool ignore_secs;
 	bool ignore_calls;
 	bool need_ident_rec;
+	bool dupes_only;
 	char *appName;
 	int dxcc;
 	bool newstation;
@@ -99,7 +102,7 @@ class TQSL_CONVERTER {
 };
 
 #if !defined(__APPLE__) && !defined(_WIN32) && !defined(__clang__)
-        #pragma GCC diagnostic ignored "-Wformat-truncation"
+#pragma GCC diagnostic ignored "-Wformat-truncation"
 #endif
 
 inline TQSL_CONVERTER::TQSL_CONVERTER()  : sentinel(0x4445) {
@@ -119,6 +122,7 @@ inline TQSL_CONVERTER::TQSL_CONVERTER()  : sentinel(0x4445) {
 	allow_dupes = true; //by default, don't change existing behavior (also helps with commit)
 	ignore_secs = false;	// Use full time
 	ignore_calls = false;	// Use calls in adif file
+	dupes_only = false;	// Not just writing tracking records
 	memset(&rec, 0, sizeof rec);
 	memset(&start, 0, sizeof start);
 	memset(&end, 0, sizeof end);
@@ -284,6 +288,15 @@ static tqsl_adifFieldDefinitions adif_qso_record_fields[] = {
 	{ "", "", TQSL_ADIF_RANGE_TYPE_NONE, 0, 0, 0, NULL },	// Correction from JJ1BDX
 };
 
+static void
+tqsl_db_get_errstr(TQSL_CONVERTER *conv) {
+	if (sqlite3_errcode(conv->seendb) == SQLITE_BUSY) {
+		strncpy(tQSL_CustomError, "The uploads database is busy. You must exit any running copies of TQSL to be able to sign a log", sizeof tQSL_CustomError);
+	} else {
+              		strncpy(tQSL_CustomError, sqlite3_errmsg(conv->seendb), sizeof tQSL_CustomError);
+	}
+}
+
 static void
 close_db(TQSL_CONVERTER *conv) {
 	tqslTrace("close_db", NULL);
@@ -292,7 +305,7 @@ close_db(TQSL_CONVERTER *conv) {
 		if (SQLITE_OK != sqlite3_exec(conv->seendb, "END;", NULL, NULL, NULL)) {
                 	tQSL_Error = TQSL_DB_ERROR;
                 	tQSL_Errno = errno;
-                	strncpy(tQSL_CustomError, sqlite3_errmsg(conv->seendb), sizeof tQSL_CustomError);
+			tqsl_db_get_errstr(conv);
                 	tqslTrace("close_db", "Error ending transaction: %s", tQSL_CustomError);
 		}
 		conv->txn = false;
@@ -301,7 +314,7 @@ close_db(TQSL_CONVERTER *conv) {
 		if (SQLITE_OK != sqlite3_close(conv->seendb)) {
                 	tQSL_Error = TQSL_DB_ERROR;
                 	tQSL_Errno = errno;
-                	strncpy(tQSL_CustomError, sqlite3_errmsg(conv->seendb), sizeof tQSL_CustomError);
+			tqsl_db_get_errstr(conv);
                 	tqslTrace("close_db", "Error closing database: %s", tQSL_CustomError);
 		}
 		// close files and clean up converters, if any
@@ -745,8 +758,7 @@ reopen:
 					if (SQLITE_OK != dbret) {
                 				tQSL_Error = TQSL_DB_ERROR;
                 				tQSL_Errno = errno;
-						snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "DB Error %s - %s",
-                						sqlite3_errmsg(conv->seendb), err_msg);
+						tqsl_db_get_errstr(conv);
                 				tqslTrace("open_db", "Error creating table: %s", tQSL_CustomError);
 						break;
 					} else {
@@ -925,9 +937,11 @@ static void parse_adif_qso(TQSL_CONVERTER *conv, int *saveErr, TQSL_ADIF_GET_FIE
 			conv->rec.band_set = true;
 			strncpy(conv->rec.band, resdata, sizeof conv->rec.band);
 		} else if (!strcasecmp(result.name, "MODE") && resdata) {
+			tqsl_strtoupper(resdata);
 			conv->rec.mode_set = true;
 			strncpy(conv->rec.mode, resdata, sizeof conv->rec.mode);
 		} else if (!strcasecmp(result.name, "SUBMODE") && resdata) {
+			tqsl_strtoupper(resdata);
 			strncpy(conv->rec.submode, resdata, sizeof conv->rec.submode);
 		} else if (!strcasecmp(result.name, "FREQ") && resdata) {
 			conv->rec.band_set = true;
@@ -941,12 +955,14 @@ static void parse_adif_qso(TQSL_CONVERTER *conv, int *saveErr, TQSL_ADIF_GET_FIE
 		} else if (!strcasecmp(result.name, "BAND_RX") && resdata) {
 			strncpy(conv->rec.rxband, resdata, sizeof conv->rec.rxband);
 		} else if (!strcasecmp(result.name, "SAT_NAME") && resdata) {
+			tqsl_strtoupper(resdata);
 			// Two-Line Elements (TLEs) call this AO-07, LoTW wants AO-7.
 			if (!strcasecmp(resdata, "AO-07"))
 				strncpy(conv->rec.satname, "AO-7", sizeof conv->rec.satname);
 			else
 				strncpy(conv->rec.satname, resdata, sizeof conv->rec.satname);
 		} else if (!strcasecmp(result.name, "PROP_MODE") && resdata) {
+			tqsl_strtoupper(resdata);
 			strncpy(conv->rec.propmode, resdata, sizeof conv->rec.propmode);
 		} else if (!strcasecmp(result.name, "QSO_DATE") && resdata) {
 			conv->rec.date_set = true;
@@ -961,17 +977,24 @@ static void parse_adif_qso(TQSL_CONVERTER *conv, int *saveErr, TQSL_ADIF_GET_FIE
 			if (conv->ignore_secs)
 				conv->rec.time.second = 0;
 		} else if (!strcasecmp(result.name, "MY_CNTY") && resdata) {
+			tqsl_strtoupper(resdata);
 			char *p = strstr(resdata, ",");			// Find the comma in "VA,Fairfax"
 			if (p) {
+				char *p1 = p;
 				*p++ = '\0';
 				strncpy(conv->rec.my_cnty_state, resdata, sizeof conv->rec.my_cnty_state);
 				while (isspace(*p)) p++;		// Skip spaces and comma
 				strncpy(conv->rec.my_county, p, sizeof conv->rec.my_county);
+				*p1 = ',';				// Put the comma back
 			} else {
 				strncpy(conv->rec.my_county, resdata, sizeof conv->rec.my_county);
 			}
 		} else if (!strcasecmp(result.name, "MY_COUNTRY") && resdata) {
-			strncpy(conv->rec.my_country, resdata, sizeof conv->rec.my_country);
+			if (strcasecmp(resdata, "United States") == 0) {
+				strncpy(conv->rec.my_country, "UNITED STATES OF AMERICA", sizeof conv->rec.my_country);
+			} else {
+				strncpy(conv->rec.my_country, resdata, sizeof conv->rec.my_country);
+			}
 		} else if (!strcasecmp(result.name, "MY_CQ_ZONE") && resdata) {
 			char *endptr;
 			long zone = strtol(resdata, &endptr, 10);
@@ -985,6 +1008,7 @@ static void parse_adif_qso(TQSL_CONVERTER *conv, int *saveErr, TQSL_ADIF_GET_FIE
 		} else if (!strcasecmp(result.name, "MY_GRIDSQUARE") && resdata) {
 			strncpy(conv->rec.my_gridsquare, resdata, sizeof conv->rec.my_gridsquare);
 		} else if (!strcasecmp(result.name, "MY_IOTA") && resdata) {
+			tqsl_strtoupper(resdata);
 			strncpy(conv->rec.my_iota, resdata, sizeof conv->rec.my_iota);
 		} else if (!strcasecmp(result.name, "MY_ITU_ZONE") && resdata) {
 			char *endptr;
@@ -995,6 +1019,7 @@ static void parse_adif_qso(TQSL_CONVERTER *conv, int *saveErr, TQSL_ADIF_GET_FIE
 				strncpy(conv->rec.my_itu_zone, resdata, sizeof conv->rec.my_itu_zone);
 			}
 		} else if (!strcasecmp(result.name, "MY_STATE") && resdata) {
+			tqsl_strtoupper(resdata);
 			strncpy(conv->rec.my_state, resdata, sizeof conv->rec.my_state);
 		} else if (!strcasecmp(result.name, "MY_VUCC_GRIDS") && resdata) {
 			strncpy(conv->rec.my_vucc_grids, resdata, sizeof conv->rec.my_vucc_grids);
@@ -1038,29 +1063,34 @@ static void parse_adif_qso(TQSL_CONVERTER *conv, int *saveErr, TQSL_ADIF_GET_FIE
 	return;
 }
 
-static int check_station(TQSL_CONVERTER *conv, const char *field, char *my, size_t len, const char *errfmt, bool update) {
+static int check_station(TQSL_CONVERTER *conv, const char *field, const char *my_field, char *my, size_t len, const char *errfmt, bool update) {
 //
 // UPDATE is a boolean that when a change is made, that change
 // is propagated to the downstream values. STATE -> COUNTY and STATE->ZONES
 //
 	char val[256];
 	char label[256];
-	bool provinceFixed = false;
-	bool oblastFixed = false;
+	const char *newProvince = NULL;
+	const char *newOblast = NULL;
 	// CA_PROVINCE can be QC but TQSL lookup expects PQ
 	if (!strcasecmp(field, "CA_PROVINCE") && !strcasecmp(my, "QC")) {
-		provinceFixed = true;
+		newProvince = "PQ";
 		strncpy(my, "PQ", len);
 	}
+	// CA_PROVINCE can be NL but TQSL lookup expects NF
+	if (!strcasecmp(field, "CA_PROVINCE") && !strcasecmp(my, "NL")) {
+		newProvince = "NF";
+		strncpy(my, "NF", len);
+	}
 
 	// RU_OBLAST can be YR but TQSL lookup expects JA
 	if (!strcasecmp(field, "RU_OBLAST") && !strcasecmp(my, "YR")) {
-		oblastFixed = true;
+		newOblast = "JA";
 		strncpy(my, "JA", len);
 	}
 	// RU_OBLAST can be YN but TQSL lookup expects JN
 	if (!strcasecmp(field, "RU_OBLAST") && !strcasecmp(my, "YN")) {
-		oblastFixed = true;
+		newOblast = "JN";
 		strncpy(my, "JN", len);
 	}
 
@@ -1076,8 +1106,8 @@ static int check_station(TQSL_CONVERTER *conv, const char *field, char *my, size
 				if (res == -1) {
 					conv->rec_done = true;
 					snprintf(tQSL_CustomError, sizeof tQSL_CustomError, errfmt, my, val);
-					tQSL_Error = TQSL_LOCATION_MISMATCH | 0x1000;
-					set_tagline(conv, field);
+					tQSL_Error = TQSL_LOCATION_MISMATCH | TQSL_MSG_FLAGGED;
+					set_tagline(conv, my_field);
 					return 1;
 				}
 				// -2 means that the label matched, so use that
@@ -1088,15 +1118,15 @@ static int check_station(TQSL_CONVERTER *conv, const char *field, char *my, size
 				conv->newstation = true;
 			} else if (strlen(val) > 0) {
 				conv->rec_done = true;
-				if (provinceFixed) {
-					strncpy(my, "QC", len);
+				if (newProvince) {
+					strncpy(my, newProvince, len);
 				}
-				if (oblastFixed) {
-					strncpy(my, "YR", len);
+				if (newOblast) {
+					strncpy(my, newOblast, len);
 				}
 				snprintf(tQSL_CustomError, sizeof tQSL_CustomError, errfmt, val, my);
 				tQSL_Error = TQSL_LOCATION_MISMATCH;
-				set_tagline(conv, field);
+				set_tagline(conv, my_field);
 				return 1;
 			} else {
 				tqsl_setLocationField(conv->loc, field, my);
@@ -1140,9 +1170,13 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 				conv->rec_done = true;
 				return 0;
 			}
-			if (stat == TQSL_ADIF_GET_FIELD_EOF)
+			if (stat == TQSL_ADIF_GET_FIELD_EOF) {
+				tqslTrace("tqsl_getConverterGABBI", "eof");
 				return 0;
+			}
+
 			if (stat != TQSL_ADIF_GET_FIELD_SUCCESS) {
+				tqslTrace("tqsl_getConverterGABBI", "adif error %d", stat);
 				tQSL_ADIF_Error = stat;
 				tQSL_Error = TQSL_ADIF_ERROR;
 				return 0;
@@ -1158,11 +1192,13 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 			if (conv->rec.my_country[0] != 0) {
 				int num_dxcc = 0;
 				tqsl_getNumDXCCEntity(&num_dxcc);
-				const char *entity;
+				const char *entity = NULL;
 				int ent_num;
+				bool found = false;
 				for (int i = 0; i < num_dxcc; i++) {
 					tqsl_getDXCCEntity(i, &ent_num, &entity);
-					if (strcasecmp(entity, conv->rec.my_country) == 0) {
+					if (entity && strcasecmp(entity, conv->rec.my_country) == 0) {
+						found = true;
 						// Consistent DXCC ?
 						if (conv->rec.my_dxcc == 0) {
 							conv->rec.my_dxcc = ent_num;
@@ -1174,6 +1210,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 								tqsl_getDXCCEntityName(conv->rec.my_dxcc, &d1);
 
 								snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "DXCC Entity|%s (%d)|%s (%d)", d1, conv->rec.my_dxcc, conv->rec.my_country, i);
+								tqslTrace("tqsl_getConverterGABBI", "cert error: %s", tQSL_CustomError);
 								tQSL_Error = TQSL_CERT_MISMATCH;
 								return 0;
 							}
@@ -1181,6 +1218,9 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 						break;
 					}
 				}
+				if (!found) {	// Country name is bogus
+					conv->rec.my_country[0] = '\0';
+				}
 			}
 			// Normalize the grids
 			if (conv->rec.my_vucc_grids[0] != 0) {
@@ -1258,30 +1298,35 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 	if (!conv->rec.callsign_set) {
 		conv->rec_done = true;
 		snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Invalid contact - QSO does not specify a Callsign");
+		tqslTrace("tqsl_getConverterGABBI", "No callsign in QSO");
 		tQSL_Error = TQSL_CUSTOM_ERROR;
 		return 0;
 	}
 	if (!conv->rec.band_set) {
 		conv->rec_done = true;
 		snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Invalid contact - QSO does not specify a band or frequency");
+		tqslTrace("tqsl_getConverterGABBI", "No band in QSO");
 		tQSL_Error = TQSL_CUSTOM_ERROR;
 		return 0;
 	}
 	if (!conv->rec.mode_set) {
 		conv->rec_done = true;
 		snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Invalid contact - QSO does not specify a mode");
+		tqslTrace("tqsl_getConverterGABBI", "No mode in QSO");
 		tQSL_Error = TQSL_CUSTOM_ERROR;
 		return 0;
 	}
 	if (!conv->rec.date_set) {
 		conv->rec_done = true;
 		snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Invalid contact - QSO does not specify a date");
+		tqslTrace("tqsl_getConverterGABBI", "No date in QSO");
 		tQSL_Error = TQSL_CUSTOM_ERROR;
 		return 0;
 	}
 	if (!conv->rec.time_set) {
 		conv->rec_done = true;
 		snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Invalid contact - QSO does not specify a time");
+		tqslTrace("tqsl_getConverterGABBI", "No time in QSO");
 		tQSL_Error = TQSL_CUSTOM_ERROR;
 		return 0;
 	}
@@ -1292,11 +1337,13 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 			conv->rec_done = true;
 			tQSL_Error = TQSL_DATE_OUT_OF_RANGE;
 			set_tagline(conv, "QSO_DATE");
+			tqslTrace("tqsl_getConverterGABBI", "QSO date out of range");
 			return 0;
 		}
 		if (tqsl_isDateValid(&(conv->end)) && tqsl_compareDates(&(conv->rec.date), &(conv->end)) > 0) {
 			conv->rec_done = true;
 			tQSL_Error = TQSL_DATE_OUT_OF_RANGE;
+			tqslTrace("tqsl_getConverterGABBI", "QSO date out of range");
 			set_tagline(conv, "QSO_DATE");
 			return 0;
 		}
@@ -1309,6 +1356,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 			conv->rec_done = true;
 			snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Invalid amateur CALL (%s)", conv->rec.callsign);
 			set_tagline(conv, "CALL");
+			tqslTrace("tqsl_getConverterGABBI", "QSO call %s invalid", conv->rec.callsign);
 			tQSL_Error = TQSL_CUSTOM_ERROR;
 			return 0;
 		}
@@ -1347,6 +1395,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 		conv->rec_done = true;
 		snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Invalid MODE (%s)", conv->rec.mode);
 		set_tagline(conv, "MODE");
+		tqslTrace("tqsl_getConverterGABBI", "QSO mode %s invalid", conv->rec.mode);
 		tQSL_Error = TQSL_CUSTOM_ERROR;
 		return 0;
 	}
@@ -1354,6 +1403,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 		conv->rec_done = true;
 		snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Invalid BAND (%s)", conv->rec.band);
 		set_tagline(conv, "BAND");
+		tqslTrace("tqsl_getConverterGABBI", "QSO band %s invalid", conv->rec.band);
 		tQSL_Error = TQSL_CUSTOM_ERROR;
 		return 0;
 	}
@@ -1361,6 +1411,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 		conv->rec_done = true;
 		snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Invalid RX BAND (%s)", conv->rec.rxband);
 		set_tagline(conv, "BAND_RX");
+		tqslTrace("tqsl_getConverterGABBI", "QSO rxband %s invalid", conv->rec.rxband);
 		tQSL_Error = TQSL_CUSTOM_ERROR;
 		return 0;
 	}
@@ -1378,6 +1429,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 		conv->rec_done = true;
 		snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Invalid PROP_MODE (%s)", conv->rec.propmode);
 		set_tagline(conv, "PROP_MODE");
+		tqslTrace("tqsl_getConverterGABBI", "QSO propmode %s invalid", conv->rec.propmode);
 		tQSL_Error = TQSL_CUSTOM_ERROR;
 		return 0;
 	}
@@ -1386,6 +1438,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 		conv->rec_done = true;
 		snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Invalid SAT_NAME (%s)", conv->rec.satname);
 		set_tagline(conv, "SAT_NAME");
+		tqslTrace("tqsl_getConverterGABBI", "QSO sat_name %s invalid", conv->rec.satname);
 		tQSL_Error = TQSL_CUSTOM_ERROR;
 		return 0;
 	}
@@ -1393,6 +1446,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 		conv->rec_done = true;
 		snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "PROP_MODE = 'SAT' but no SAT_NAME");
 		set_tagline(conv, "PROP_MODE");
+		tqslTrace("tqsl_getConverterGABBI", "QSO prop_mode sat but no sat_name");
 		tQSL_Error = TQSL_CUSTOM_ERROR;
 		return 0;
 	}
@@ -1400,6 +1454,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 		conv->rec_done = true;
 		snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "SAT_NAME set but PROP_MODE is not 'SAT'");
 		set_tagline(conv, "SAT_NAME");
+		tqslTrace("tqsl_getConverterGABBI", "QSO sat_name set but no prop_mode");
 		tQSL_Error = TQSL_CUSTOM_ERROR;
 		return 0;
 	}
@@ -1408,6 +1463,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 	if (conv->location_handling != TQSL_LOC_UPDATE && conv->ncerts <= 0) {
 		conv->rec_done = true;
 		tQSL_Error = TQSL_CERT_NOT_FOUND;
+		tqslTrace("tqsl_getConverterGABBI", "Cert not found");
 		return 0;
 	}
 
@@ -1427,6 +1483,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 				snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Callsign|%s|%s", conv->callsign, conv->rec.my_call);
 				if (!set_tagline(conv, "STATION_CALLSIGN"))
 					set_tagline(conv, "OPERATOR");
+				tqslTrace("tqsl_getConverterGABBI", "my_call %s", tQSL_CustomError);
 				tQSL_Error = TQSL_CERT_MISMATCH;
 				return 0;
 			}
@@ -1434,7 +1491,8 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 	}
 
 	// Lookup cert - start with conv->dxcc
-	int targetdxcc = conv->dxcc;
+	int targetdxcc;
+	targetdxcc = conv->dxcc;
 
 	// If we're in update mode, use the DXCC from the log
 	if (conv->location_handling == TQSL_LOC_UPDATE) {
@@ -1444,13 +1502,21 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 	}
 
 	bool anyfound;
-	int cidx = find_matching_cert(conv, targetdxcc, &anyfound);
+	int cidx;
+	cidx = find_matching_cert(conv, targetdxcc, &anyfound);
 	if (cidx < 0) {
 		conv->rec_done = true;
-		const char *entName;
-		tqsl_getDXCCEntityName(targetdxcc, &entName);
+		const char *entName = NULL;
+		if (tqsl_getDXCCEntityName(targetdxcc, &entName) || entName == NULL) {
+			set_tagline(conv, "MY_DXCC");
+			snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "MY_DXCC|%d", conv->rec.my_dxcc);
+			conv->rec.my_dxcc = 0;
+			tqslTrace("tqsl_getConverterGABBI", "my_dxcc %s", tQSL_CustomError);
+			tQSL_Error = TQSL_INVALID_ADIF;
+			return 0;
+		}
 		snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "%s|%s", conv->callsign, entName);
-		tQSL_Error = TQSL_CERT_NOT_FOUND | 0x1000;
+		tQSL_Error = TQSL_CERT_NOT_FOUND | TQSL_MSG_FLAGGED;
 		if (anyfound) {
 			tQSL_Error = TQSL_CERT_DATE_MISMATCH;
 			set_tagline(conv, "QSO_DATE");
@@ -1497,6 +1563,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 				snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Callsign|%s|%s", conv->callsign, conv->rec.my_call);
 				if (!set_tagline(conv, "STATION_CALLSIGN"))
 					set_tagline(conv, "OPERATOR");
+				tqslTrace("tqsl_getConverterGABBI", "operator error %s", tQSL_CustomError);
 				tQSL_Error = TQSL_CERT_MISMATCH;
 				return 0;
 			}
@@ -1510,12 +1577,13 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 					tqsl_updateStationLocationCapture(conv->loc);
 				} else {
 					conv->rec_done = true;
-					const char *d1, *d2;
+					const char *d1, *d2 = NULL;
 					tqsl_getDXCCEntityName(conv->dxcc, &d1);
 					tqsl_getDXCCEntityName(conv->rec.my_dxcc, &d2);
 
-					snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "DXCC Entity|%s (%d)|%s (%d)", d1, conv->dxcc, d2, conv->rec.my_dxcc);
+					snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "DXCC Entity|%s (%d)|%s (%d)", d1 ? d1 : "Invalid", conv->dxcc, d2 ? d2 : "Invalid", conv->rec.my_dxcc);
 					set_tagline(conv, "MY_DXCC");
+					tqslTrace("tqsl_getConverterGABBI", "my_dxcc error %s", tQSL_CustomError);
 					tQSL_Error = TQSL_CERT_MISMATCH;
 					return 0;
 				}
@@ -1537,6 +1605,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 			unsigned int stnLen = strlen(val);
 			unsigned int logLen = strlen(conv->rec.my_gridsquare);
 			unsigned int compareLen = (stnLen < logLen ? stnLen : logLen);
+
 			if (strstr(val, ",") || strstr(conv->rec.my_gridsquare, ",")) {	// If it's a corner/edge
 				vector<string>stngrids;
 				vector<string>qsogrids;
@@ -1554,15 +1623,22 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 							compareLen = 99; // Doesn't match, so error out if appropriate
 							break;
 						}
+                                        	compareLen = 0;                 // Matches.
 					}
-					compareLen = 0;			// Matches.
 				}
-			}
+                        }
 			if (conv->location_handling == TQSL_LOC_UPDATE) {
 				okgrid = (strcasecmp(conv->rec.my_gridsquare, val) == 0);
 			} else {
 				okgrid = (compareLen == 0 || strncasecmp(conv->rec.my_gridsquare, val, compareLen) == 0);
 			}
+
+			/*
+ 			 * FT8 four-char grid handling.
+			 * For station location set to AA01aa and FT8 saying AA01
+			 * Treat that as a match
+			 */
+			okgrid = (compareLen == 0 || strncasecmp(conv->rec.my_gridsquare, val, compareLen) == 0);
 			if (!okgrid) {
 				if (conv->location_handling == TQSL_LOC_UPDATE) {
 					tqsl_setLocationField(conv->loc, "GRIDSQUARE", conv->rec.my_gridsquare);
@@ -1575,60 +1651,101 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 						conv->rec_done = true;
 						snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Gridsquare|%s|%s", val, conv->rec.my_gridsquare);
 						set_tagline(conv, "GRIDSQUARE");
+						tqslTrace("tqsl_getConverterGABBI", "incorrect gridsquare error %s", tQSL_CustomError);
 						tQSL_Error = TQSL_LOCATION_MISMATCH;
 						return 0;
 					}
 				}
 			}
+			if (!okgrid) {
+				if (conv->location_handling == TQSL_LOC_UPDATE) {
+					tqsl_setLocationField(conv->loc, "GRIDSQUARE", conv->rec.my_gridsquare);
+					conv->newstation = true;
+				} else {
+					if (val[0] == '\0') {           // If station location has an empty grid
+						tqsl_setLocationField(conv->loc, "GRIDSQUARE", conv->rec.my_gridsquare);
+						conv->newstation = true;
+					} else {
+						conv->rec_done = true;
+						snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "Gridsquare|%s|%s", val, conv->rec.my_gridsquare);
+						set_tagline(conv, "GRIDSQUARE");
+						tQSL_Error = TQSL_LOCATION_MISMATCH;
+						return 0;
+					}
+				}
+			}
+
 		}
 
 		switch (conv->dxcc) {
 			case 6:		// Alaska
 			case 110:	// Hawaii
 			case 291:	// Cont US
-				if (check_station(conv, "US_STATE", conv->rec.my_state, sizeof conv->rec.my_state, "US State|%s|%s", true)) return 0;
-				if (check_station(conv, "US_COUNTY", conv->rec.my_county, sizeof conv->rec.my_county, "US County|%s|%s", false)) return 0;
+				if (check_station(conv, "US_STATE", "MY_STATE", conv->rec.my_state, sizeof conv->rec.my_state, "US State|%s|%s", true)) return 0;
+				if (check_station(conv, "US_COUNTY", "MY_CNTY", conv->rec.my_county, sizeof conv->rec.my_county, "US County|%s|%s", false)) return 0;
 				break;
 			case 1:		// Canada
-				if (check_station(conv, "CA_PROVINCE", conv->rec.my_state, sizeof conv->rec.my_state, "CA Province|%s|%s", true)) return 0;
+				if (check_station(conv, "CA_PROVINCE", "MY_STATE", conv->rec.my_state, sizeof conv->rec.my_state, "CA Province|%s|%s", true)) return 0;
 				break;
 			case 15:	// Asiatic Russia
 			case 54:	// European Russia
 			case 61:	// FJL
 			case 125:	// Juan Fernandez
 			case 151:	// Malyj Vysotskij
-				if (check_station(conv, "RU_OBLAST", conv->rec.my_state, sizeof conv->rec.my_state, "RU Oblast|%s|%s", true)) return 0;
+				if (check_station(conv, "RU_OBLAST", "MY_STATE", conv->rec.my_state, sizeof conv->rec.my_state, "RU Oblast|%s|%s", true)) return 0;
 				break;
 			case 318:	// China
-				if (check_station(conv, "CN_PROVINCE", conv->rec.my_state, sizeof conv->rec.my_state, "CN Province|%s|%s", true)) return 0;
+				if (check_station(conv, "CN_PROVINCE", "MY_STATE", conv->rec.my_state, sizeof conv->rec.my_state, "CN Province|%s|%s", true)) return 0;
 				break;
 			case 150:	// Australia
-				if (check_station(conv, "AU_STATE", conv->rec.my_state, sizeof conv->rec.my_state, "AU State|%s|%s", true)) return 0;
+				if (check_station(conv, "AU_STATE", "MY_STATE", conv->rec.my_state, sizeof conv->rec.my_state, "AU State|%s|%s", true)) return 0;
 				break;
 			case 339:	// Japan
-				if (check_station(conv, "JA_PREFECTURE", conv->rec.my_state, sizeof conv->rec.my_state, "JA Prefecture|%s|%s", true)) return 0;
-				if (check_station(conv, "JA_CITY_GUN_KU", conv->rec.my_county, sizeof conv->rec.my_county, "JA City/Gun/Ku|%s|%s", false)) return 0;
+				if (check_station(conv, "JA_PREFECTURE", "MY_STATE", conv->rec.my_state, sizeof conv->rec.my_state, "JA Prefecture|%s|%s", true)) return 0;
+				if (check_station(conv, "JA_CITY_GUN_KU", "MY_CNTY", conv->rec.my_county, sizeof conv->rec.my_county, "JA City/Gun/Ku|%s|%s", false)) return 0;
 				break;
 			case 5:		// Finland
-				if (check_station(conv, "FI_KUNTA", conv->rec.my_state, sizeof conv->rec.my_state, "FI Kunta|%s|%s", true)) return 0;
+				if (check_station(conv, "FI_KUNTA", "MY_STATE", conv->rec.my_state, sizeof conv->rec.my_state, "FI Kunta|%s|%s", true)) return 0;
 				break;
 		}
 
-		if (check_station(conv, "ITUZ", conv->rec.my_itu_zone, sizeof conv->rec.my_itu_zone, "ITU Zone|%s|%s", false)) return 0;
-		if (check_station(conv, "CQZ", conv->rec.my_cq_zone, sizeof conv->rec.my_cq_zone, "CQ Zone|%s|%s", false)) return 0;
-		if (check_station(conv, "IOTA", conv->rec.my_iota, sizeof conv->rec.my_iota, "IOTA|%s|%s", false)) return 0;
-
+		if (check_station(conv, "ITUZ", "MY_ITU_ZONE", conv->rec.my_itu_zone, sizeof conv->rec.my_itu_zone, "ITU Zone|%s|%s", false)) return 0;
+		if (check_station(conv, "CQZ", "MY_CQ_ZONE", conv->rec.my_cq_zone, sizeof conv->rec.my_cq_zone, "CQ Zone|%s|%s", false)) return 0;
+		if (conv->rec.my_iota[0] != '\0') {
+			// IOTA identifiers: AF-123 form.
+			int num = strtol(conv->rec.my_iota + 3, NULL, 10);
+			if ((num == 0) ||
+			    (strlen(conv->rec.my_iota) != 6) ||
+			    (strncmp(conv->rec.my_iota, "AF-", 3) &&
+			     strncmp(conv->rec.my_iota, "AN-", 3) &&
+			     strncmp(conv->rec.my_iota, "AS-", 3) &&
+			     strncmp(conv->rec.my_iota, "EU-", 3) &&
+			     strncmp(conv->rec.my_iota, "NA-", 3) &&
+			     strncmp(conv->rec.my_iota, "OC-", 3) &&
+			     strncmp(conv->rec.my_iota, "SA-", 3))) {
+				conv->rec_done = true;
+				snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "MY_IOTA|%s", conv->rec.my_iota);
+				tQSL_Error = TQSL_INVALID_ADIF;
+				set_tagline(conv, "MY_IOTA");
+				tqslTrace("tqsl_getConverterGABBI", "my_iota error %s", tQSL_CustomError);
+				return 0;
+			}
+			if (check_station(conv, "IOTA", "MY_IOTA", conv->rec.my_iota, sizeof conv->rec.my_iota, "IOTA|%s|%s", false)) return 0;
+		}
+		char locstate[5];
+		locstate[0] = '\0';
 		if (conv->rec.my_cnty_state[0] != '\0') {
-			char locstate[5];
 			tqsl_getLocationField(conv->loc, "US_STATE", locstate, sizeof locstate);
 			if (strcasecmp(conv->rec.my_cnty_state, locstate)) {		// County does not match state
 				conv->rec_done = true;
 				snprintf(tQSL_CustomError, sizeof tQSL_CustomError, "US County State|%s|%s", conv->rec.my_cnty_state, locstate);
 				set_tagline(conv, "US_STATE");
-				tQSL_Error = TQSL_LOCATION_MISMATCH | 0x1000;
+				tqslTrace("tqsl_getConverterGABBI", "us_state error %s", tQSL_CustomError);
+				tQSL_Error = TQSL_LOCATION_MISMATCH | TQSL_MSG_FLAGGED;
 				return 0;
 			}
 		}
+
 		if (conv->newstation) {
 			conv->newstation = false;
 			conv->loc_uid++;
@@ -1643,7 +1760,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 
 	const char *grec = tqsl_getGABBItCONTACTData(conv->certs[conv->cert_idx], conv->loc, &(conv->rec),
 		conv->loc_uid, signdata, sizeof(signdata));
-	if (grec) {
+	if (conv->dupes_only || grec) {
 		conv->rec_done = true;
 		if (!conv->allow_dupes) {
 			char stnloc[128];
@@ -1685,13 +1802,14 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 						remove_db(conv->dbpath);
 						free(conv->dbpath);
 					}
+					tqsl_db_get_errstr(conv);
 					tQSL_Error = TQSL_DB_ERROR;
 					return 0;
 				}
 				return 0;
 			} else if (rc < 0) {
 				//non-zero return, but not "not found" - thus error
-				strncpy(tQSL_CustomError, sqlite3_errmsg(conv->seendb), sizeof tQSL_CustomError);
+				tqsl_db_get_errstr(conv);
 				if (SQLITE_NOTADB == rc) {		// This isn't a database
 					close_db(conv);
 					remove_db(conv->dbpath);
@@ -1764,7 +1882,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 				del_dbrec(conv->seendb, dupekey);
 			} else if (rc < 0) {
 				//non-zero return, but not "not found" - thus error
-				strncpy(tQSL_CustomError, sqlite3_errmsg(conv->seendb), sizeof tQSL_CustomError);
+				tqsl_db_get_errstr(conv);
 				if (SQLITE_NOTADB == rc) {		// Not a database
 					close_db(conv);
 					remove_db(conv->dbpath);
@@ -1778,7 +1896,7 @@ tqsl_getConverterGABBI(tQSL_Converter convp) {
 			int dbput_err;
 			dbput_err = put_dbrec(conv->seendb, dupekey, reinterpret_cast<const char *>(stnloc));
 			if (0 != dbput_err) {
-				strncpy(tQSL_CustomError, sqlite3_errmsg(conv->seendb), sizeof tQSL_CustomError);
+				tqsl_db_get_errstr(conv);
 				if (SQLITE_NOTADB == dbput_err) {
 					close_db(conv);
 					remove_db(conv->dbpath);
@@ -1883,6 +2001,15 @@ tqsl_setConverterAppName(tQSL_Converter convp, const char *app) {
 	return 0;
 }
 
+DLLEXPORT int CALLCONVENTION
+tqsl_setConverterDupesOnly(tQSL_Converter convp, int dupesOnly) {
+	TQSL_CONVERTER *conv;
+	if (!(conv = check_conv(convp)))
+		return 1;
+	conv->dupes_only = (dupesOnly != 0);
+	return 0;
+}
+
 DLLEXPORT int CALLCONVENTION
 tqsl_setConverterQTHDetails(tQSL_Converter convp, int logverify) {
 	TQSL_CONVERTER *conv;
@@ -1957,7 +2084,7 @@ tqsl_getDuplicateRecords(tQSL_Converter convp, char *key, char *data, int keylen
     	}
 	const unsigned char* result = sqlite3_column_text(conv->bulk_read, 1);
 	if (!result) {
-		strncpy(tQSL_CustomError, sqlite3_errmsg(conv->seendb), sizeof tQSL_CustomError);
+		tqsl_db_get_errstr(conv);
 		tQSL_Error = TQSL_DB_ERROR;
 		tQSL_Errno = errno;
 		return 1;
@@ -1992,21 +2119,21 @@ tqsl_getDuplicateRecordsV2(tQSL_Converter convp, char *key, char *data, int keyl
 	}
 	if (SQLITE_ROW != rc) {
 		sqlite3_finalize(conv->bulk_read);
-		strncpy(tQSL_CustomError, sqlite3_errmsg(conv->seendb), sizeof tQSL_CustomError);
+		tqsl_db_get_errstr(conv);
 		tQSL_Error = TQSL_DB_ERROR;
 		tQSL_Errno = errno;
 		return 1;
     	}
 	const unsigned char* rkey = sqlite3_column_text(conv->bulk_read, 0);
 	if (!rkey) {
-		strncpy(tQSL_CustomError, sqlite3_errmsg(conv->seendb), sizeof tQSL_CustomError);
+		tqsl_db_get_errstr(conv);
 		tQSL_Error = TQSL_DB_ERROR;
 		tQSL_Errno = errno;
 		return 1;
 	}
 	const unsigned char* rdata = sqlite3_column_text(conv->bulk_read, 1);
 	if (!rdata) {
-		strncpy(tQSL_CustomError, sqlite3_errmsg(conv->seendb), sizeof tQSL_CustomError);
+		tqsl_db_get_errstr(conv);
 		tQSL_Error = TQSL_DB_ERROR;
 		tQSL_Errno = errno;
 		return 1;
@@ -2037,7 +2164,7 @@ tqsl_putDuplicateRecord(tQSL_Converter convp, const char *key, const char *data,
 	int status = put_dbrec(conv->seendb, key, data);
 
 	if (0 != status) {
-		strncpy(tQSL_CustomError, sqlite3_errmsg(conv->seendb), sizeof tQSL_CustomError);
+		tqsl_db_get_errstr(conv);
 		tQSL_Error = TQSL_DB_ERROR;
 		tQSL_Errno = errno;
 		return 1;
