@@ -15,18 +15,21 @@
 #include <exception>
 #include "tqsllib.h"
 
-class myexc : public std::exception {
-public:
-	myexc(const std::string& err) : std::exception() { _err = err; }
+using std::exception;
+using std::string;
+
+class myexc : public exception {
+ public:
+	explicit myexc(const string& err) : exception() { _err = err; }
 	myexc(const myexc& x) { _err = x._err; }
 	virtual const char *what() const throw () { return _err.c_str(); }
-	virtual ~myexc() throw () {}
-private:
-	std::string _err;
+	virtual ~myexc() throw() {}
+ private:
+	string _err;
 };
 
 class tqslexc : public myexc {
-public:
+ public:
 	tqslexc() : myexc(tqsl_getErrorString()) {}
 };
 
