@@ -28,6 +28,8 @@
 	#include <limits.h>
 #endif
 
+#include <cstdio>
+
 #include "adif.h"
 #include "cabrillo.h"
 
@@ -36,7 +38,7 @@
   */
 
 #ifndef PATH_MAX				// Should be set by <limits.h>
-#define PATH_MAX 4096
+#define PATH_MAX 4096	///< Override in case not defined in limits.h
 #endif
 
 /* Sizes */
@@ -75,9 +77,12 @@
 #define TQSL_CERT_CB_PROMPT          0		///< Callback prompt
 #define TQSL_CERT_CB_DUPLICATE       0x100	///< Dupe cert callback
 #define TQSL_CERT_CB_ERROR           0x200	///< Error import callback
-#define TQSL_CERT_CB_LOADED          0x300	///< Cert loaded callback
+#define TQSL_CERT_CB_LOADED          0x400	///< Cert loaded callback
+#define TQSL_CERT_CB_SERIAL          0x800	///< User cert serial callback
 #define TQSL_CERT_CB_RESULT_TYPE(x)  ((x) & 0x0f00) ///< Result type mask
 
+#define TQSL_MSG_FLAGGED             0x1000	///< Alt message handling flag
+
 typedef void * tQSL_Cert;		///< Opaque certificate type
 typedef void * tQSL_Location;		///< Opaque location type
 typedef char * tQSL_StationDataEnc;	///< Opaque station data type
@@ -411,6 +416,10 @@ DLLEXPORT int CALLCONVENTION tqsl_isCertificateSuperceded(tQSL_Cert cert, int *s
   */
 DLLEXPORT int CALLCONVENTION tqsl_getCertificateKeyOnly(tQSL_Cert cert, int *keyonly);
 
+/** Find out if the "certificate" is renewable
+  */
+DLLEXPORT int CALLCONVENTION tqsl_isCertificateRenewable(tQSL_Cert cert, int *status);
+
 /** Get the encoded certificate for inclusion in a GABBI file.
   */
 DLLEXPORT int CALLCONVENTION tqsl_getCertificateEncoded(tQSL_Cert cert, char *buf, int bufsiz);
@@ -749,6 +758,16 @@ DLLEXPORT int CALLCONVENTION tqsl_createCertRequest(const char *filename, TQSL_C
   */
 DLLEXPORT int CALLCONVENTION tqsl_exportPKCS12File(tQSL_Cert cert, const char *filename, const char *p12password);
 
+/** Save a key pair and certificates to a file in PKCS12 format.
+  * Use downgraded crypto for Apple Keyring compatibility.
+  *
+  * The tQSL_Cert must be initialized for signing (see tqsl_beginSigning())
+  * if the user certificate is being exported.
+  *
+  * The supplied \c p12password is used to encrypt the PKCS12 data.
+  */
+DLLEXPORT int CALLCONVENTION tqsl_exportPKCS12FileWeakCrypto(tQSL_Cert cert, const char *filename, const char *p12password);
+
 /** Save a key pair and certificates to a Base64 string in PKCS12 format.
   *
   * The tQSL_Cert must be initialized for signing (see tqsl_beginSigning())
@@ -1213,6 +1232,12 @@ DLLEXPORT int CALLCONVENTION tqsl_saveCallsignLocationInfo(const char *callsign,
 /** Retrieve the json results for a given callsign location Detail. */
 DLLEXPORT int CALLCONVENTION tqsl_getCallsignLocationInfo(const char *callsign, char **buf);
 
+#define TQSL_VALID_VUCC_ENT 0x2000		///< Grid is valid for DXCC Entity
+#define TQSL_VALID_VUCC_PAS 0x4000		///< Grid is valid for Primary Administrative Subdivision (State, etc.)
+
+/** Validate that a given four-character gridsquare is acceptable for entity and primary administrative subdivision */
+DLLEXPORT int CALLCONVENTION tqsl_validateVUCCGrid(int entity, const char *pas, const char *grid, int *result);
+
 /** Get the number of DXCC entities in the primary DXCC list.
   */
 DLLEXPORT int CALLCONVENTION tqsl_getNumDXCCEntity(int *number);
@@ -1403,6 +1428,19 @@ DLLEXPORT const char* CALLCONVENTION tqsl_getGABBItCONTACT(tQSL_Cert cert, tQSL_
 DLLEXPORT const char* CALLCONVENTION tqsl_getGABBItCONTACTData(tQSL_Cert cert, tQSL_Location loc, TQSL_QSO_RECORD *qso,
 	int stationuid, char *signdata, int sdlen);
 
+#define GRID_ERROR_INVALID_FIELD 2		///< Field - first two chars - is invalid
+#define GRID_ERROR_INVALID_SQUARE 3		///< Square - second two chars - is invalid
+#define GRID_ERROR_INVALID_SUBSQUARE 4		///< Subsquare - third pair - is invalid
+#define GRID_ERROR_INVALID_SUBSUBSQUARE 5	///< Sub-subsquare - fourth pair etc. - is invalid
+#define GRID_ERROR_INVALID_FORMAT 6		///< Format error
+
+/** Check and normalize a Maidenhead Gridsquare
+  * \li \c grid is the input string
+  * \li \c twelve is true if 12 character grids are allowed
+  * \li \c newGrid gets the resulting edited grid
+ */
+DLLEXPORT int CALLCONVENTION tqsl_verifyGridFormat(const char *grid, int twelve, char* newGrid, int newlen);
+
 /** @} */
 
 /** Output to a diagnostic trace file (if one is opened.
